package net.sf.corn.converter.json;

import java.util.regex.Pattern;

import net.sf.corn.converter.ConversionException;
import net.sf.corn.converter.ObjectReferenceMap;
/**
 * Converter for {@link java.util.regex.Pattern} class
 * @author Serhat Dirik
 */
class CnvPattern extends AbstractJsonConverter {
	private static final String REGEX_ATTRIBUTE = "regex";
	private static final String FLAGS = "flags";
	
	CnvPattern() {
		getConvertibles().add(java.util.regex.Pattern.class);
	}

	@Override
	protected JsTypeObject inToTarget(Object o, ObjectReferenceMap referenceMap)
			throws ConversionException {
		JsTypeComplex jsonObject = new JsTypeComplex(o.getClass());
		JsonConverter transformer = new JsonConverter();
		Pattern pt = (Pattern) o;
		jsonObject.put(REGEX_ATTRIBUTE,	transformer.toTargetType(pt.toString(), referenceMap));
		jsonObject.put( FLAGS,	transformer.toTargetType(pt.flags(), referenceMap));
		return jsonObject;
	}

	@Override
	protected Object inToJava(Class<?> clazz, JsTypeObject obj)
			throws ConversionException {
		
		if (!obj.isComplex()) {
			throw new ConversionException("Only complex type is allowed");
		}
		JsTypeComplex jso = (JsTypeComplex) obj;
		if ( !jso.has(REGEX_ATTRIBUTE)){
			throw new ConversionException(REGEX_ATTRIBUTE + " attribute not found");
		}
		if ( !jso.has(FLAGS)){
			throw new ConversionException(FLAGS + " attribute not found");
		}
		JsonConverter transformer = new JsonConverter();
		String regex = (String) transformer.toJava(jso.get(REGEX_ATTRIBUTE));
		Integer flags = (Integer) transformer.toJava(jso.get(FLAGS));
		return Pattern.compile(regex, flags);
	}

	@Override
	public boolean isSubjectForCircularReferenceCheck() {
		return false;
	}
}
